# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _cproton
else:
    import _cproton

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



def pn_message_encode(msg, BIN_OUT):
    return _cproton.pn_message_encode(msg, BIN_OUT)

def pn_message_decode(msg, BIN_IN):
    return _cproton.pn_message_decode(msg, BIN_IN)

def pn_link_send(transport, BIN_IN):
    return _cproton.pn_link_send(transport, BIN_IN)

def pn_link_recv(link, BIN_OUT):
    return _cproton.pn_link_recv(link, BIN_OUT)

def pn_transport_push(transport, BIN_IN):
    return _cproton.pn_transport_push(transport, BIN_IN)

def pn_transport_peek(transport, BIN_OUT):
    return _cproton.pn_transport_peek(transport, BIN_OUT)

def pn_delivery(link, STRING):
    return _cproton.pn_delivery(link, STRING)

def pn_delivery_tag(delivery):
    return _cproton.pn_delivery_tag(delivery)

def pn_data_decode(data, BIN_IN):
    return _cproton.pn_data_decode(data, BIN_IN)

def pn_data_encode(data, BIN_OUT):
    return _cproton.pn_data_encode(data, BIN_OUT)

def pn_data_format(data, VTEXT_OUT):
    return _cproton.pn_data_format(data, VTEXT_OUT)

def pn_ssl_get_cipher_name(ssl, OUTPUT):
    return _cproton.pn_ssl_get_cipher_name(ssl, OUTPUT)

def pn_ssl_get_protocol_name(ssl, OUTPUT):
    return _cproton.pn_ssl_get_protocol_name(ssl, OUTPUT)

def pn_ssl_get_remote_subject_subfield(ssl, field):
    return _cproton.pn_ssl_get_remote_subject_subfield(ssl, field)

def pn_ssl_get_cert_fingerprint(ssl, OUTPUT, hash_alg):
    return _cproton.pn_ssl_get_cert_fingerprint(ssl, OUTPUT, hash_alg)

def pn_ssl_get_peer_hostname(ssl, VTEXT_OUT):
    return _cproton.pn_ssl_get_peer_hostname(ssl, VTEXT_OUT)

def pn_pyref_incref(object):
    return _cproton.pn_pyref_incref(object)

def pn_pyref_decref(object):
    return _cproton.pn_pyref_decref(object)

def pn_pyref_refcount(object):
    return _cproton.pn_pyref_refcount(object)

def pn_py2void(object):
    return _cproton.pn_py2void(object)

def pn_void2py(object):
    return _cproton.pn_void2py(object)

def pn_cast_pn_void(object):
    return _cproton.pn_cast_pn_void(object)

def pn_pytracer(transport, message):
    return _cproton.pn_pytracer(transport, message)

def pn_transport_set_pytracer(transport, obj):
    return _cproton.pn_transport_set_pytracer(transport, obj)

def pn_transport_get_pytracer(transport):
    return _cproton.pn_transport_get_pytracer(transport)
PROTON_IMPORT_EXPORT_H = _cproton.PROTON_IMPORT_EXPORT_H
PN_VERSION_MAJOR = _cproton.PN_VERSION_MAJOR
PN_VERSION_MINOR = _cproton.PN_VERSION_MINOR
PN_VERSION_POINT = _cproton.PN_VERSION_POINT
PROTON_TYPES_H = _cproton.PROTON_TYPES_H
PN_MILLIS_MAX = _cproton.PN_MILLIS_MAX

def pn_bytes(size, start):
    return _cproton.pn_bytes(size, start)

def pn_rwbytes(size, start):
    return _cproton.pn_rwbytes(size, start)
PROTON_OBJECT_H = _cproton.PROTON_OBJECT_H

def pn_class_create(name, initialize, finalize, incref, decref, refcount):
    return _cproton.pn_class_create(name, initialize, finalize, incref, decref, refcount)

def pn_class_new(clazz, size):
    return _cproton.pn_class_new(clazz, size)

def pn_class_name(clazz):
    return _cproton.pn_class_name(clazz)

def pn_class_id(clazz):
    return _cproton.pn_class_id(clazz)

def pn_class(object):
    return _cproton.pn_class(object)

def pn_incref(object):
    return _cproton.pn_incref(object)

def pn_decref(object):
    return _cproton.pn_decref(object)

def pn_refcount(object):
    return _cproton.pn_refcount(object)

def pn_free(object):
    return _cproton.pn_free(object)

def pn_tostring(object):
    return _cproton.pn_tostring(object)

def pn_record():
    return _cproton.pn_record()

def pn_record_def(record, key, clazz):
    return _cproton.pn_record_def(record, key, clazz)

def pn_record_has(record, key):
    return _cproton.pn_record_has(record, key)

def pn_record_get(record, key):
    return _cproton.pn_record_get(record, key)

def pn_record_set(record, key, value):
    return _cproton.pn_record_set(record, key, value)

def pn_record_clear(record):
    return _cproton.pn_record_clear(record)
PROTON_ERROR_H = _cproton.PROTON_ERROR_H
PN_OK = _cproton.PN_OK
PN_EOS = _cproton.PN_EOS
PN_ERR = _cproton.PN_ERR
PN_OVERFLOW = _cproton.PN_OVERFLOW
PN_UNDERFLOW = _cproton.PN_UNDERFLOW
PN_STATE_ERR = _cproton.PN_STATE_ERR
PN_ARG_ERR = _cproton.PN_ARG_ERR
PN_TIMEOUT = _cproton.PN_TIMEOUT
PN_INTR = _cproton.PN_INTR
PN_INPROGRESS = _cproton.PN_INPROGRESS
PN_OUT_OF_MEMORY = _cproton.PN_OUT_OF_MEMORY
PN_ABORTED = _cproton.PN_ABORTED

def pn_code(code):
    return _cproton.pn_code(code)

def pn_error():
    return _cproton.pn_error()

def pn_error_free(error):
    return _cproton.pn_error_free(error)

def pn_error_clear(error):
    return _cproton.pn_error_clear(error)

def pn_error_set(error, code, text):
    return _cproton.pn_error_set(error, code, text)

def pn_error_code(error):
    return _cproton.pn_error_code(error)

def pn_error_text(error):
    return _cproton.pn_error_text(error)

def pn_error_copy(error, src):
    return _cproton.pn_error_copy(error, src)
PROTON_CONDITION_H = _cproton.PROTON_CONDITION_H

def pn_condition_is_set(condition):
    return _cproton.pn_condition_is_set(condition)

def pn_condition_clear(condition):
    return _cproton.pn_condition_clear(condition)

def pn_condition_get_name(condition):
    return _cproton.pn_condition_get_name(condition)

def pn_condition_set_name(condition, name):
    return _cproton.pn_condition_set_name(condition, name)

def pn_condition_get_description(condition):
    return _cproton.pn_condition_get_description(condition)

def pn_condition_set_description(condition, description):
    return _cproton.pn_condition_set_description(condition, description)

def pn_condition_info(condition):
    return _cproton.pn_condition_info(condition)

def pn_condition_is_redirect(condition):
    return _cproton.pn_condition_is_redirect(condition)

def pn_condition_redirect_host(condition):
    return _cproton.pn_condition_redirect_host(condition)

def pn_condition_redirect_port(condition):
    return _cproton.pn_condition_redirect_port(condition)

def pn_condition_copy(dest, src):
    return _cproton.pn_condition_copy(dest, src)

def pn_condition():
    return _cproton.pn_condition()

def pn_condition_free(arg1):
    return _cproton.pn_condition_free(arg1)
PROTON_CONNECTION_H = _cproton.PROTON_CONNECTION_H
PN_LOCAL_UNINIT = _cproton.PN_LOCAL_UNINIT
PN_LOCAL_ACTIVE = _cproton.PN_LOCAL_ACTIVE
PN_LOCAL_CLOSED = _cproton.PN_LOCAL_CLOSED
PN_REMOTE_UNINIT = _cproton.PN_REMOTE_UNINIT
PN_REMOTE_ACTIVE = _cproton.PN_REMOTE_ACTIVE
PN_REMOTE_CLOSED = _cproton.PN_REMOTE_CLOSED
PN_LOCAL_MASK = _cproton.PN_LOCAL_MASK
PN_REMOTE_MASK = _cproton.PN_REMOTE_MASK

def pn_connection():
    return _cproton.pn_connection()

def pn_connection_free(connection):
    return _cproton.pn_connection_free(connection)

def pn_connection_release(connection):
    return _cproton.pn_connection_release(connection)

def pn_connection_error(connection):
    return _cproton.pn_connection_error(connection)

def pn_connection_collect(connection, collector):
    return _cproton.pn_connection_collect(connection, collector)

def pn_connection_collector(connection):
    return _cproton.pn_connection_collector(connection)

def pn_connection_get_context(connection):
    return _cproton.pn_connection_get_context(connection)

def pn_connection_set_context(connection, context):
    return _cproton.pn_connection_set_context(connection, context)

def pn_connection_attachments(connection):
    return _cproton.pn_connection_attachments(connection)

def pn_connection_state(connection):
    return _cproton.pn_connection_state(connection)

def pn_connection_open(connection):
    return _cproton.pn_connection_open(connection)

def pn_connection_close(connection):
    return _cproton.pn_connection_close(connection)

def pn_connection_reset(connection):
    return _cproton.pn_connection_reset(connection)

def pn_connection_condition(connection):
    return _cproton.pn_connection_condition(connection)

def pn_connection_remote_condition(connection):
    return _cproton.pn_connection_remote_condition(connection)

def pn_connection_get_container(connection):
    return _cproton.pn_connection_get_container(connection)

def pn_connection_set_container(connection, container):
    return _cproton.pn_connection_set_container(connection, container)

def pn_connection_set_user(connection, user):
    return _cproton.pn_connection_set_user(connection, user)

def pn_connection_set_password(connection, password):
    return _cproton.pn_connection_set_password(connection, password)

def pn_connection_set_authorization(connection, authzid):
    return _cproton.pn_connection_set_authorization(connection, authzid)

def pn_connection_get_user(connection):
    return _cproton.pn_connection_get_user(connection)

def pn_connection_get_authorization(connection):
    return _cproton.pn_connection_get_authorization(connection)

def pn_connection_get_hostname(connection):
    return _cproton.pn_connection_get_hostname(connection)

def pn_connection_set_hostname(connection, hostname):
    return _cproton.pn_connection_set_hostname(connection, hostname)

def pn_connection_remote_container(connection):
    return _cproton.pn_connection_remote_container(connection)

def pn_connection_remote_hostname(connection):
    return _cproton.pn_connection_remote_hostname(connection)

def pn_connection_offered_capabilities(connection):
    return _cproton.pn_connection_offered_capabilities(connection)

def pn_connection_desired_capabilities(connection):
    return _cproton.pn_connection_desired_capabilities(connection)

def pn_connection_properties(connection):
    return _cproton.pn_connection_properties(connection)

def pn_connection_remote_offered_capabilities(connection):
    return _cproton.pn_connection_remote_offered_capabilities(connection)

def pn_connection_remote_desired_capabilities(connection):
    return _cproton.pn_connection_remote_desired_capabilities(connection)

def pn_connection_remote_properties(connection):
    return _cproton.pn_connection_remote_properties(connection)

def pn_connection_transport(connection):
    return _cproton.pn_connection_transport(connection)
PROTON_SESSION_H = _cproton.PROTON_SESSION_H

def pn_session(connection):
    return _cproton.pn_session(connection)

def pn_session_free(session):
    return _cproton.pn_session_free(session)

def pn_session_get_context(session):
    return _cproton.pn_session_get_context(session)

def pn_session_set_context(session, context):
    return _cproton.pn_session_set_context(session, context)

def pn_session_attachments(session):
    return _cproton.pn_session_attachments(session)

def pn_session_state(session):
    return _cproton.pn_session_state(session)

def pn_session_error(session):
    return _cproton.pn_session_error(session)

def pn_session_condition(session):
    return _cproton.pn_session_condition(session)

def pn_session_remote_condition(session):
    return _cproton.pn_session_remote_condition(session)

def pn_session_connection(session):
    return _cproton.pn_session_connection(session)

def pn_session_open(session):
    return _cproton.pn_session_open(session)

def pn_session_close(session):
    return _cproton.pn_session_close(session)

def pn_session_get_incoming_capacity(session):
    return _cproton.pn_session_get_incoming_capacity(session)

def pn_session_set_incoming_capacity(session, capacity):
    return _cproton.pn_session_set_incoming_capacity(session, capacity)

def pn_session_get_outgoing_window(session):
    return _cproton.pn_session_get_outgoing_window(session)

def pn_session_set_outgoing_window(session, window):
    return _cproton.pn_session_set_outgoing_window(session, window)

def pn_session_outgoing_bytes(session):
    return _cproton.pn_session_outgoing_bytes(session)

def pn_session_incoming_bytes(session):
    return _cproton.pn_session_incoming_bytes(session)

def pn_session_head(connection, state):
    return _cproton.pn_session_head(connection, state)

def pn_session_next(session, state):
    return _cproton.pn_session_next(session, state)
PROTON_LINK_H = _cproton.PROTON_LINK_H

def pn_sender(session, name):
    return _cproton.pn_sender(session, name)

def pn_receiver(session, name):
    return _cproton.pn_receiver(session, name)

def pn_link_free(link):
    return _cproton.pn_link_free(link)

def pn_link_get_context(link):
    return _cproton.pn_link_get_context(link)

def pn_link_set_context(link, context):
    return _cproton.pn_link_set_context(link, context)

def pn_link_attachments(link):
    return _cproton.pn_link_attachments(link)

def pn_link_name(link):
    return _cproton.pn_link_name(link)

def pn_link_is_sender(link):
    return _cproton.pn_link_is_sender(link)

def pn_link_is_receiver(link):
    return _cproton.pn_link_is_receiver(link)

def pn_link_state(link):
    return _cproton.pn_link_state(link)

def pn_link_error(link):
    return _cproton.pn_link_error(link)

def pn_link_condition(link):
    return _cproton.pn_link_condition(link)

def pn_link_remote_condition(link):
    return _cproton.pn_link_remote_condition(link)

def pn_link_session(link):
    return _cproton.pn_link_session(link)

def pn_link_head(connection, state):
    return _cproton.pn_link_head(connection, state)

def pn_link_next(link, state):
    return _cproton.pn_link_next(link, state)

def pn_link_open(link):
    return _cproton.pn_link_open(link)

def pn_link_close(link):
    return _cproton.pn_link_close(link)

def pn_link_detach(link):
    return _cproton.pn_link_detach(link)

def pn_link_source(link):
    return _cproton.pn_link_source(link)

def pn_link_target(link):
    return _cproton.pn_link_target(link)

def pn_link_remote_source(link):
    return _cproton.pn_link_remote_source(link)

def pn_link_remote_target(link):
    return _cproton.pn_link_remote_target(link)

def pn_link_current(link):
    return _cproton.pn_link_current(link)

def pn_link_advance(link):
    return _cproton.pn_link_advance(link)

def pn_link_credit(link):
    return _cproton.pn_link_credit(link)

def pn_link_queued(link):
    return _cproton.pn_link_queued(link)

def pn_link_remote_credit(link):
    return _cproton.pn_link_remote_credit(link)

def pn_link_get_drain(link):
    return _cproton.pn_link_get_drain(link)

def pn_link_drained(link):
    return _cproton.pn_link_drained(link)

def pn_link_available(link):
    return _cproton.pn_link_available(link)
PN_SND_UNSETTLED = _cproton.PN_SND_UNSETTLED
PN_SND_SETTLED = _cproton.PN_SND_SETTLED
PN_SND_MIXED = _cproton.PN_SND_MIXED
PN_RCV_FIRST = _cproton.PN_RCV_FIRST
PN_RCV_SECOND = _cproton.PN_RCV_SECOND

def pn_link_snd_settle_mode(link):
    return _cproton.pn_link_snd_settle_mode(link)

def pn_link_rcv_settle_mode(link):
    return _cproton.pn_link_rcv_settle_mode(link)

def pn_link_set_snd_settle_mode(link, mode):
    return _cproton.pn_link_set_snd_settle_mode(link, mode)

def pn_link_set_rcv_settle_mode(link, mode):
    return _cproton.pn_link_set_rcv_settle_mode(link, mode)

def pn_link_remote_snd_settle_mode(link):
    return _cproton.pn_link_remote_snd_settle_mode(link)

def pn_link_remote_rcv_settle_mode(link):
    return _cproton.pn_link_remote_rcv_settle_mode(link)

def pn_link_unsettled(link):
    return _cproton.pn_link_unsettled(link)

def pn_unsettled_head(link):
    return _cproton.pn_unsettled_head(link)

def pn_unsettled_next(delivery):
    return _cproton.pn_unsettled_next(delivery)

def pn_link_offered(sender, credit):
    return _cproton.pn_link_offered(sender, credit)

def pn_link_flow(receiver, credit):
    return _cproton.pn_link_flow(receiver, credit)

def pn_link_drain(receiver, credit):
    return _cproton.pn_link_drain(receiver, credit)

def pn_link_set_drain(receiver, drain):
    return _cproton.pn_link_set_drain(receiver, drain)

def pn_link_draining(receiver):
    return _cproton.pn_link_draining(receiver)

def pn_link_max_message_size(link):
    return _cproton.pn_link_max_message_size(link)

def pn_link_set_max_message_size(link, size):
    return _cproton.pn_link_set_max_message_size(link, size)

def pn_link_remote_max_message_size(link):
    return _cproton.pn_link_remote_max_message_size(link)

def pn_link_properties(link):
    return _cproton.pn_link_properties(link)

def pn_link_remote_properties(link):
    return _cproton.pn_link_remote_properties(link)
PROTON_TERMINUS_H = _cproton.PROTON_TERMINUS_H
PN_UNSPECIFIED = _cproton.PN_UNSPECIFIED
PN_SOURCE = _cproton.PN_SOURCE
PN_TARGET = _cproton.PN_TARGET
PN_COORDINATOR = _cproton.PN_COORDINATOR
PN_NONDURABLE = _cproton.PN_NONDURABLE
PN_CONFIGURATION = _cproton.PN_CONFIGURATION
PN_DELIVERIES = _cproton.PN_DELIVERIES
PN_EXPIRE_WITH_LINK = _cproton.PN_EXPIRE_WITH_LINK
PN_EXPIRE_WITH_SESSION = _cproton.PN_EXPIRE_WITH_SESSION
PN_EXPIRE_WITH_CONNECTION = _cproton.PN_EXPIRE_WITH_CONNECTION
PN_EXPIRE_NEVER = _cproton.PN_EXPIRE_NEVER
PN_DIST_MODE_UNSPECIFIED = _cproton.PN_DIST_MODE_UNSPECIFIED
PN_DIST_MODE_COPY = _cproton.PN_DIST_MODE_COPY
PN_DIST_MODE_MOVE = _cproton.PN_DIST_MODE_MOVE

def pn_terminus_get_type(terminus):
    return _cproton.pn_terminus_get_type(terminus)

def pn_terminus_set_type(terminus, type):
    return _cproton.pn_terminus_set_type(terminus, type)

def pn_terminus_get_address(terminus):
    return _cproton.pn_terminus_get_address(terminus)

def pn_terminus_set_address(terminus, address):
    return _cproton.pn_terminus_set_address(terminus, address)

def pn_terminus_get_distribution_mode(terminus):
    return _cproton.pn_terminus_get_distribution_mode(terminus)

def pn_terminus_set_distribution_mode(terminus, mode):
    return _cproton.pn_terminus_set_distribution_mode(terminus, mode)

def pn_terminus_get_durability(terminus):
    return _cproton.pn_terminus_get_durability(terminus)

def pn_terminus_set_durability(terminus, durability):
    return _cproton.pn_terminus_set_durability(terminus, durability)

def pn_terminus_get_expiry_policy(terminus):
    return _cproton.pn_terminus_get_expiry_policy(terminus)

def pn_terminus_has_expiry_policy(terminus):
    return _cproton.pn_terminus_has_expiry_policy(terminus)

def pn_terminus_set_expiry_policy(terminus, policy):
    return _cproton.pn_terminus_set_expiry_policy(terminus, policy)

def pn_terminus_get_timeout(terminus):
    return _cproton.pn_terminus_get_timeout(terminus)

def pn_terminus_set_timeout(terminus, timeout):
    return _cproton.pn_terminus_set_timeout(terminus, timeout)

def pn_terminus_is_dynamic(terminus):
    return _cproton.pn_terminus_is_dynamic(terminus)

def pn_terminus_set_dynamic(terminus, dynamic):
    return _cproton.pn_terminus_set_dynamic(terminus, dynamic)

def pn_terminus_properties(terminus):
    return _cproton.pn_terminus_properties(terminus)

def pn_terminus_capabilities(terminus):
    return _cproton.pn_terminus_capabilities(terminus)

def pn_terminus_outcomes(terminus):
    return _cproton.pn_terminus_outcomes(terminus)

def pn_terminus_filter(terminus):
    return _cproton.pn_terminus_filter(terminus)

def pn_terminus_copy(terminus, src):
    return _cproton.pn_terminus_copy(terminus, src)
PROTON_DELIVERY_H = _cproton.PROTON_DELIVERY_H

def pn_delivery_get_context(delivery):
    return _cproton.pn_delivery_get_context(delivery)

def pn_delivery_set_context(delivery, context):
    return _cproton.pn_delivery_set_context(delivery, context)

def pn_delivery_attachments(delivery):
    return _cproton.pn_delivery_attachments(delivery)

def pn_delivery_link(delivery):
    return _cproton.pn_delivery_link(delivery)

def pn_delivery_local(delivery):
    return _cproton.pn_delivery_local(delivery)

def pn_delivery_local_state(delivery):
    return _cproton.pn_delivery_local_state(delivery)

def pn_delivery_remote(delivery):
    return _cproton.pn_delivery_remote(delivery)

def pn_delivery_remote_state(delivery):
    return _cproton.pn_delivery_remote_state(delivery)

def pn_delivery_settled(delivery):
    return _cproton.pn_delivery_settled(delivery)

def pn_delivery_pending(delivery):
    return _cproton.pn_delivery_pending(delivery)

def pn_delivery_partial(delivery):
    return _cproton.pn_delivery_partial(delivery)

def pn_delivery_aborted(delivery):
    return _cproton.pn_delivery_aborted(delivery)

def pn_delivery_writable(delivery):
    return _cproton.pn_delivery_writable(delivery)

def pn_delivery_readable(delivery):
    return _cproton.pn_delivery_readable(delivery)

def pn_delivery_updated(delivery):
    return _cproton.pn_delivery_updated(delivery)

def pn_delivery_update(delivery, state):
    return _cproton.pn_delivery_update(delivery, state)

def pn_delivery_clear(delivery):
    return _cproton.pn_delivery_clear(delivery)

def pn_delivery_current(delivery):
    return _cproton.pn_delivery_current(delivery)

def pn_delivery_abort(delivery):
    return _cproton.pn_delivery_abort(delivery)

def pn_delivery_settle(delivery):
    return _cproton.pn_delivery_settle(delivery)

def pn_delivery_dump(delivery):
    return _cproton.pn_delivery_dump(delivery)

def pn_delivery_buffered(delivery):
    return _cproton.pn_delivery_buffered(delivery)

def pn_work_head(connection):
    return _cproton.pn_work_head(connection)

def pn_work_next(delivery):
    return _cproton.pn_work_next(delivery)
PROTON_DISPOSITION_H = _cproton.PROTON_DISPOSITION_H
PN_RECEIVED = _cproton.PN_RECEIVED
PN_ACCEPTED = _cproton.PN_ACCEPTED
PN_REJECTED = _cproton.PN_REJECTED
PN_RELEASED = _cproton.PN_RELEASED
PN_MODIFIED = _cproton.PN_MODIFIED

def pn_disposition_type(disposition):
    return _cproton.pn_disposition_type(disposition)

def pn_disposition_type_name(disposition_type):
    return _cproton.pn_disposition_type_name(disposition_type)

def pn_disposition_condition(disposition):
    return _cproton.pn_disposition_condition(disposition)

def pn_disposition_data(disposition):
    return _cproton.pn_disposition_data(disposition)

def pn_disposition_get_section_number(disposition):
    return _cproton.pn_disposition_get_section_number(disposition)

def pn_disposition_set_section_number(disposition, section_number):
    return _cproton.pn_disposition_set_section_number(disposition, section_number)

def pn_disposition_get_section_offset(disposition):
    return _cproton.pn_disposition_get_section_offset(disposition)

def pn_disposition_set_section_offset(disposition, section_offset):
    return _cproton.pn_disposition_set_section_offset(disposition, section_offset)

def pn_disposition_is_failed(disposition):
    return _cproton.pn_disposition_is_failed(disposition)

def pn_disposition_set_failed(disposition, failed):
    return _cproton.pn_disposition_set_failed(disposition, failed)

def pn_disposition_is_undeliverable(disposition):
    return _cproton.pn_disposition_is_undeliverable(disposition)

def pn_disposition_set_undeliverable(disposition, undeliverable):
    return _cproton.pn_disposition_set_undeliverable(disposition, undeliverable)

def pn_disposition_annotations(disposition):
    return _cproton.pn_disposition_annotations(disposition)
PROTON_TRANSPORT_H = _cproton.PROTON_TRANSPORT_H
PN_TRACE_OFF = _cproton.PN_TRACE_OFF
PN_TRACE_RAW = _cproton.PN_TRACE_RAW
PN_TRACE_FRM = _cproton.PN_TRACE_FRM
PN_TRACE_DRV = _cproton.PN_TRACE_DRV
PN_TRACE_EVT = _cproton.PN_TRACE_EVT

def pn_transport():
    return _cproton.pn_transport()

def pn_transport_set_server(transport):
    return _cproton.pn_transport_set_server(transport)

def pn_transport_free(transport):
    return _cproton.pn_transport_free(transport)

def pn_transport_get_user(transport):
    return _cproton.pn_transport_get_user(transport)

def pn_transport_require_auth(transport, required):
    return _cproton.pn_transport_require_auth(transport, required)

def pn_transport_is_authenticated(transport):
    return _cproton.pn_transport_is_authenticated(transport)

def pn_transport_require_encryption(transport, required):
    return _cproton.pn_transport_require_encryption(transport, required)

def pn_transport_is_encrypted(transport):
    return _cproton.pn_transport_is_encrypted(transport)

def pn_transport_condition(transport):
    return _cproton.pn_transport_condition(transport)

def pn_transport_logger(transport):
    return _cproton.pn_transport_logger(transport)

def pn_transport_error(transport):
    return _cproton.pn_transport_error(transport)

def pn_transport_bind(transport, connection):
    return _cproton.pn_transport_bind(transport, connection)

def pn_transport_unbind(transport):
    return _cproton.pn_transport_unbind(transport)

def pn_transport_trace(transport, trace):
    return _cproton.pn_transport_trace(transport, trace)

def pn_transport_set_tracer(transport, tracer):
    return _cproton.pn_transport_set_tracer(transport, tracer)

def pn_transport_get_tracer(transport):
    return _cproton.pn_transport_get_tracer(transport)

def pn_transport_get_context(transport):
    return _cproton.pn_transport_get_context(transport)

def pn_transport_set_context(transport, context):
    return _cproton.pn_transport_set_context(transport, context)

def pn_transport_attachments(transport):
    return _cproton.pn_transport_attachments(transport)

def pn_transport_log(transport, message):
    return _cproton.pn_transport_log(transport, message)

def pn_transport_logf(*args):
    return _cproton.pn_transport_logf(*args)

def pn_transport_get_channel_max(transport):
    return _cproton.pn_transport_get_channel_max(transport)

def pn_transport_set_channel_max(transport, channel_max):
    return _cproton.pn_transport_set_channel_max(transport, channel_max)

def pn_transport_remote_channel_max(transport):
    return _cproton.pn_transport_remote_channel_max(transport)

def pn_transport_get_max_frame(transport):
    return _cproton.pn_transport_get_max_frame(transport)

def pn_transport_set_max_frame(transport, size):
    return _cproton.pn_transport_set_max_frame(transport, size)

def pn_transport_get_remote_max_frame(transport):
    return _cproton.pn_transport_get_remote_max_frame(transport)

def pn_transport_get_idle_timeout(transport):
    return _cproton.pn_transport_get_idle_timeout(transport)

def pn_transport_set_idle_timeout(transport, timeout):
    return _cproton.pn_transport_set_idle_timeout(transport, timeout)

def pn_transport_get_remote_idle_timeout(transport):
    return _cproton.pn_transport_get_remote_idle_timeout(transport)

def pn_transport_input(transport, bytes, available):
    return _cproton.pn_transport_input(transport, bytes, available)

def pn_transport_output(transport, bytes, size):
    return _cproton.pn_transport_output(transport, bytes, size)

def pn_transport_capacity(transport):
    return _cproton.pn_transport_capacity(transport)

def pn_transport_tail(transport):
    return _cproton.pn_transport_tail(transport)

def pn_transport_process(transport, size):
    return _cproton.pn_transport_process(transport, size)

def pn_transport_close_tail(transport):
    return _cproton.pn_transport_close_tail(transport)

def pn_transport_pending(transport):
    return _cproton.pn_transport_pending(transport)

def pn_transport_head(transport):
    return _cproton.pn_transport_head(transport)

def pn_transport_pop(transport, size):
    return _cproton.pn_transport_pop(transport, size)

def pn_transport_close_head(transport):
    return _cproton.pn_transport_close_head(transport)

def pn_transport_quiesced(transport):
    return _cproton.pn_transport_quiesced(transport)

def pn_transport_head_closed(transport):
    return _cproton.pn_transport_head_closed(transport)

def pn_transport_tail_closed(transport):
    return _cproton.pn_transport_tail_closed(transport)

def pn_transport_closed(transport):
    return _cproton.pn_transport_closed(transport)

def pn_transport_tick(transport, now):
    return _cproton.pn_transport_tick(transport, now)

def pn_transport_get_frames_output(transport):
    return _cproton.pn_transport_get_frames_output(transport)

def pn_transport_get_frames_input(transport):
    return _cproton.pn_transport_get_frames_input(transport)

def pn_transport_connection(transport):
    return _cproton.pn_transport_connection(transport)
PROTON_EVENT_H = _cproton.PROTON_EVENT_H
PN_EVENT_NONE = _cproton.PN_EVENT_NONE
PN_REACTOR_INIT = _cproton.PN_REACTOR_INIT
PN_REACTOR_QUIESCED = _cproton.PN_REACTOR_QUIESCED
PN_REACTOR_FINAL = _cproton.PN_REACTOR_FINAL
PN_TIMER_TASK = _cproton.PN_TIMER_TASK
PN_CONNECTION_INIT = _cproton.PN_CONNECTION_INIT
PN_CONNECTION_BOUND = _cproton.PN_CONNECTION_BOUND
PN_CONNECTION_UNBOUND = _cproton.PN_CONNECTION_UNBOUND
PN_CONNECTION_LOCAL_OPEN = _cproton.PN_CONNECTION_LOCAL_OPEN
PN_CONNECTION_REMOTE_OPEN = _cproton.PN_CONNECTION_REMOTE_OPEN
PN_CONNECTION_LOCAL_CLOSE = _cproton.PN_CONNECTION_LOCAL_CLOSE
PN_CONNECTION_REMOTE_CLOSE = _cproton.PN_CONNECTION_REMOTE_CLOSE
PN_CONNECTION_FINAL = _cproton.PN_CONNECTION_FINAL
PN_SESSION_INIT = _cproton.PN_SESSION_INIT
PN_SESSION_LOCAL_OPEN = _cproton.PN_SESSION_LOCAL_OPEN
PN_SESSION_REMOTE_OPEN = _cproton.PN_SESSION_REMOTE_OPEN
PN_SESSION_LOCAL_CLOSE = _cproton.PN_SESSION_LOCAL_CLOSE
PN_SESSION_REMOTE_CLOSE = _cproton.PN_SESSION_REMOTE_CLOSE
PN_SESSION_FINAL = _cproton.PN_SESSION_FINAL
PN_LINK_INIT = _cproton.PN_LINK_INIT
PN_LINK_LOCAL_OPEN = _cproton.PN_LINK_LOCAL_OPEN
PN_LINK_REMOTE_OPEN = _cproton.PN_LINK_REMOTE_OPEN
PN_LINK_LOCAL_CLOSE = _cproton.PN_LINK_LOCAL_CLOSE
PN_LINK_REMOTE_CLOSE = _cproton.PN_LINK_REMOTE_CLOSE
PN_LINK_LOCAL_DETACH = _cproton.PN_LINK_LOCAL_DETACH
PN_LINK_REMOTE_DETACH = _cproton.PN_LINK_REMOTE_DETACH
PN_LINK_FLOW = _cproton.PN_LINK_FLOW
PN_LINK_FINAL = _cproton.PN_LINK_FINAL
PN_DELIVERY = _cproton.PN_DELIVERY
PN_TRANSPORT = _cproton.PN_TRANSPORT
PN_TRANSPORT_AUTHENTICATED = _cproton.PN_TRANSPORT_AUTHENTICATED
PN_TRANSPORT_ERROR = _cproton.PN_TRANSPORT_ERROR
PN_TRANSPORT_HEAD_CLOSED = _cproton.PN_TRANSPORT_HEAD_CLOSED
PN_TRANSPORT_TAIL_CLOSED = _cproton.PN_TRANSPORT_TAIL_CLOSED
PN_TRANSPORT_CLOSED = _cproton.PN_TRANSPORT_CLOSED
PN_SELECTABLE_INIT = _cproton.PN_SELECTABLE_INIT
PN_SELECTABLE_UPDATED = _cproton.PN_SELECTABLE_UPDATED
PN_SELECTABLE_READABLE = _cproton.PN_SELECTABLE_READABLE
PN_SELECTABLE_WRITABLE = _cproton.PN_SELECTABLE_WRITABLE
PN_SELECTABLE_ERROR = _cproton.PN_SELECTABLE_ERROR
PN_SELECTABLE_EXPIRED = _cproton.PN_SELECTABLE_EXPIRED
PN_SELECTABLE_FINAL = _cproton.PN_SELECTABLE_FINAL
PN_CONNECTION_WAKE = _cproton.PN_CONNECTION_WAKE
PN_LISTENER_ACCEPT = _cproton.PN_LISTENER_ACCEPT
PN_LISTENER_CLOSE = _cproton.PN_LISTENER_CLOSE
PN_PROACTOR_INTERRUPT = _cproton.PN_PROACTOR_INTERRUPT
PN_PROACTOR_TIMEOUT = _cproton.PN_PROACTOR_TIMEOUT
PN_PROACTOR_INACTIVE = _cproton.PN_PROACTOR_INACTIVE
PN_LISTENER_OPEN = _cproton.PN_LISTENER_OPEN
PN_RAW_CONNECTION_CONNECTED = _cproton.PN_RAW_CONNECTION_CONNECTED
PN_RAW_CONNECTION_CLOSED_READ = _cproton.PN_RAW_CONNECTION_CLOSED_READ
PN_RAW_CONNECTION_CLOSED_WRITE = _cproton.PN_RAW_CONNECTION_CLOSED_WRITE
PN_RAW_CONNECTION_DISCONNECTED = _cproton.PN_RAW_CONNECTION_DISCONNECTED
PN_RAW_CONNECTION_NEED_READ_BUFFERS = _cproton.PN_RAW_CONNECTION_NEED_READ_BUFFERS
PN_RAW_CONNECTION_NEED_WRITE_BUFFERS = _cproton.PN_RAW_CONNECTION_NEED_WRITE_BUFFERS
PN_RAW_CONNECTION_READ = _cproton.PN_RAW_CONNECTION_READ
PN_RAW_CONNECTION_WRITTEN = _cproton.PN_RAW_CONNECTION_WRITTEN
PN_RAW_CONNECTION_WAKE = _cproton.PN_RAW_CONNECTION_WAKE
PN_RAW_CONNECTION_DRAIN_BUFFERS = _cproton.PN_RAW_CONNECTION_DRAIN_BUFFERS

def pn_event_type_name(type):
    return _cproton.pn_event_type_name(type)

def pn_collector():
    return _cproton.pn_collector()

def pn_collector_free(collector):
    return _cproton.pn_collector_free(collector)

def pn_collector_release(collector):
    return _cproton.pn_collector_release(collector)

def pn_collector_drain(collector):
    return _cproton.pn_collector_drain(collector)

def pn_collector_put(collector, clazz, context, type):
    return _cproton.pn_collector_put(collector, clazz, context, type)

def pn_collector_put_object(collector, object, type):
    return _cproton.pn_collector_put_object(collector, object, type)

def pn_collector_peek(collector):
    return _cproton.pn_collector_peek(collector)

def pn_collector_pop(collector):
    return _cproton.pn_collector_pop(collector)

def pn_collector_next(collector):
    return _cproton.pn_collector_next(collector)

def pn_collector_prev(collector):
    return _cproton.pn_collector_prev(collector)

def pn_collector_more(collector):
    return _cproton.pn_collector_more(collector)

def pn_event_type(event):
    return _cproton.pn_event_type(event)

def pn_event_class(event):
    return _cproton.pn_event_class(event)

def pn_event_context(event):
    return _cproton.pn_event_context(event)

def pn_event_connection(event):
    return _cproton.pn_event_connection(event)

def pn_event_session(event):
    return _cproton.pn_event_session(event)

def pn_event_link(event):
    return _cproton.pn_event_link(event)

def pn_event_delivery(event):
    return _cproton.pn_event_delivery(event)

def pn_event_transport(event):
    return _cproton.pn_event_transport(event)

def pn_event_attachments(event):
    return _cproton.pn_event_attachments(event)

def pn_event_condition(event):
    return _cproton.pn_event_condition(event)

def pn_event_copy(evt):
    return _cproton.pn_event_copy(evt)
PROTON_MESSAGE_H = _cproton.PROTON_MESSAGE_H
PN_DEFAULT_PRIORITY = _cproton.PN_DEFAULT_PRIORITY

def pn_message():
    return _cproton.pn_message()

def pn_message_free(msg):
    return _cproton.pn_message_free(msg)

def pn_message_clear(msg):
    return _cproton.pn_message_clear(msg)

def pn_message_errno(msg):
    return _cproton.pn_message_errno(msg)

def pn_message_error(msg):
    return _cproton.pn_message_error(msg)

def pn_message_is_inferred(msg):
    return _cproton.pn_message_is_inferred(msg)

def pn_message_set_inferred(msg, inferred):
    return _cproton.pn_message_set_inferred(msg, inferred)

def pn_message_is_durable(msg):
    return _cproton.pn_message_is_durable(msg)

def pn_message_set_durable(msg, durable):
    return _cproton.pn_message_set_durable(msg, durable)

def pn_message_get_priority(msg):
    return _cproton.pn_message_get_priority(msg)

def pn_message_set_priority(msg, priority):
    return _cproton.pn_message_set_priority(msg, priority)

def pn_message_get_ttl(msg):
    return _cproton.pn_message_get_ttl(msg)

def pn_message_set_ttl(msg, ttl):
    return _cproton.pn_message_set_ttl(msg, ttl)

def pn_message_is_first_acquirer(msg):
    return _cproton.pn_message_is_first_acquirer(msg)

def pn_message_set_first_acquirer(msg, first):
    return _cproton.pn_message_set_first_acquirer(msg, first)

def pn_message_get_delivery_count(msg):
    return _cproton.pn_message_get_delivery_count(msg)

def pn_message_set_delivery_count(msg, count):
    return _cproton.pn_message_set_delivery_count(msg, count)

def pn_message_get_id(msg):
    return _cproton.pn_message_get_id(msg)

def pn_message_set_id(msg, id):
    return _cproton.pn_message_set_id(msg, id)

def pn_message_get_user_id(msg):
    return _cproton.pn_message_get_user_id(msg)

def pn_message_set_user_id(msg, user_id):
    return _cproton.pn_message_set_user_id(msg, user_id)

def pn_message_get_address(msg):
    return _cproton.pn_message_get_address(msg)

def pn_message_set_address(msg, address):
    return _cproton.pn_message_set_address(msg, address)

def pn_message_get_subject(msg):
    return _cproton.pn_message_get_subject(msg)

def pn_message_set_subject(msg, subject):
    return _cproton.pn_message_set_subject(msg, subject)

def pn_message_get_reply_to(msg):
    return _cproton.pn_message_get_reply_to(msg)

def pn_message_set_reply_to(msg, reply_to):
    return _cproton.pn_message_set_reply_to(msg, reply_to)

def pn_message_get_correlation_id(msg):
    return _cproton.pn_message_get_correlation_id(msg)

def pn_message_set_correlation_id(msg, id):
    return _cproton.pn_message_set_correlation_id(msg, id)

def pn_message_get_content_type(msg):
    return _cproton.pn_message_get_content_type(msg)

def pn_message_set_content_type(msg, type):
    return _cproton.pn_message_set_content_type(msg, type)

def pn_message_get_content_encoding(msg):
    return _cproton.pn_message_get_content_encoding(msg)

def pn_message_set_content_encoding(msg, encoding):
    return _cproton.pn_message_set_content_encoding(msg, encoding)

def pn_message_get_expiry_time(msg):
    return _cproton.pn_message_get_expiry_time(msg)

def pn_message_set_expiry_time(msg, time):
    return _cproton.pn_message_set_expiry_time(msg, time)

def pn_message_get_creation_time(msg):
    return _cproton.pn_message_get_creation_time(msg)

def pn_message_set_creation_time(msg, time):
    return _cproton.pn_message_set_creation_time(msg, time)

def pn_message_get_group_id(msg):
    return _cproton.pn_message_get_group_id(msg)

def pn_message_set_group_id(msg, group_id):
    return _cproton.pn_message_set_group_id(msg, group_id)

def pn_message_get_group_sequence(msg):
    return _cproton.pn_message_get_group_sequence(msg)

def pn_message_set_group_sequence(msg, n):
    return _cproton.pn_message_set_group_sequence(msg, n)

def pn_message_get_reply_to_group_id(msg):
    return _cproton.pn_message_get_reply_to_group_id(msg)

def pn_message_set_reply_to_group_id(msg, reply_to_group_id):
    return _cproton.pn_message_set_reply_to_group_id(msg, reply_to_group_id)

def pn_message_instructions(msg):
    return _cproton.pn_message_instructions(msg)

def pn_message_annotations(msg):
    return _cproton.pn_message_annotations(msg)

def pn_message_properties(msg):
    return _cproton.pn_message_properties(msg)

def pn_message_body(msg):
    return _cproton.pn_message_body(msg)

def pn_message_encode2(msg, buf):
    return _cproton.pn_message_encode2(msg, buf)

def pn_message_send(msg, sender, buf):
    return _cproton.pn_message_send(msg, sender, buf)

def pn_message_data(msg, data):
    return _cproton.pn_message_data(msg, data)
PROTON_SASL_H = _cproton.PROTON_SASL_H
PN_SASL_NONE = _cproton.PN_SASL_NONE
PN_SASL_OK = _cproton.PN_SASL_OK
PN_SASL_AUTH = _cproton.PN_SASL_AUTH
PN_SASL_SYS = _cproton.PN_SASL_SYS
PN_SASL_PERM = _cproton.PN_SASL_PERM
PN_SASL_TEMP = _cproton.PN_SASL_TEMP

def pn_sasl(transport):
    return _cproton.pn_sasl(transport)

def pn_sasl_extended():
    return _cproton.pn_sasl_extended()

def pn_sasl_done(sasl, outcome):
    return _cproton.pn_sasl_done(sasl, outcome)

def pn_sasl_outcome(sasl):
    return _cproton.pn_sasl_outcome(sasl)

def pn_sasl_get_user(sasl):
    return _cproton.pn_sasl_get_user(sasl)

def pn_sasl_get_authorization(sasl):
    return _cproton.pn_sasl_get_authorization(sasl)

def pn_sasl_get_mech(sasl):
    return _cproton.pn_sasl_get_mech(sasl)

def pn_sasl_allowed_mechs(sasl, mechs):
    return _cproton.pn_sasl_allowed_mechs(sasl, mechs)

def pn_sasl_set_allow_insecure_mechs(sasl, insecure):
    return _cproton.pn_sasl_set_allow_insecure_mechs(sasl, insecure)

def pn_sasl_get_allow_insecure_mechs(sasl):
    return _cproton.pn_sasl_get_allow_insecure_mechs(sasl)

def pn_sasl_config_name(sasl, name):
    return _cproton.pn_sasl_config_name(sasl, name)

def pn_sasl_config_path(sasl, path):
    return _cproton.pn_sasl_config_path(sasl, path)
PROTON_SSL_H = _cproton.PROTON_SSL_H
PN_SSL_MODE_CLIENT = _cproton.PN_SSL_MODE_CLIENT
PN_SSL_MODE_SERVER = _cproton.PN_SSL_MODE_SERVER
PN_SSL_RESUME_UNKNOWN = _cproton.PN_SSL_RESUME_UNKNOWN
PN_SSL_RESUME_NEW = _cproton.PN_SSL_RESUME_NEW
PN_SSL_RESUME_REUSED = _cproton.PN_SSL_RESUME_REUSED

def pn_ssl_present():
    return _cproton.pn_ssl_present()

def pn_ssl_domain(mode):
    return _cproton.pn_ssl_domain(mode)

def pn_ssl_domain_free(domain):
    return _cproton.pn_ssl_domain_free(domain)

def pn_ssl_domain_set_credentials(domain, credential_1, credential_2, password):
    return _cproton.pn_ssl_domain_set_credentials(domain, credential_1, credential_2, password)

def pn_ssl_domain_set_trusted_ca_db(domain, certificate_db):
    return _cproton.pn_ssl_domain_set_trusted_ca_db(domain, certificate_db)
PN_SSL_VERIFY_NULL = _cproton.PN_SSL_VERIFY_NULL
PN_SSL_VERIFY_PEER = _cproton.PN_SSL_VERIFY_PEER
PN_SSL_ANONYMOUS_PEER = _cproton.PN_SSL_ANONYMOUS_PEER
PN_SSL_VERIFY_PEER_NAME = _cproton.PN_SSL_VERIFY_PEER_NAME

def pn_ssl_domain_set_peer_authentication(domain, mode, trusted_CAs):
    return _cproton.pn_ssl_domain_set_peer_authentication(domain, mode, trusted_CAs)

def pn_ssl_domain_set_protocols(domain, protocols):
    return _cproton.pn_ssl_domain_set_protocols(domain, protocols)

def pn_ssl_domain_set_ciphers(domain, ciphers):
    return _cproton.pn_ssl_domain_set_ciphers(domain, ciphers)

def pn_ssl_domain_allow_unsecured_client(domain):
    return _cproton.pn_ssl_domain_allow_unsecured_client(domain)

def pn_ssl(transport):
    return _cproton.pn_ssl(transport)

def pn_ssl_init(ssl, domain, session_id):
    return _cproton.pn_ssl_init(ssl, domain, session_id)

def pn_ssl_get_ssf(ssl):
    return _cproton.pn_ssl_get_ssf(ssl)

def pn_ssl_resume_status(ssl):
    return _cproton.pn_ssl_resume_status(ssl)

def pn_ssl_set_peer_hostname(ssl, hostname):
    return _cproton.pn_ssl_set_peer_hostname(ssl, hostname)

def pn_ssl_get_remote_subject(ssl):
    return _cproton.pn_ssl_get_remote_subject(ssl)
PN_SSL_CERT_SUBJECT_COUNTRY_NAME = _cproton.PN_SSL_CERT_SUBJECT_COUNTRY_NAME
PN_SSL_CERT_SUBJECT_STATE_OR_PROVINCE = _cproton.PN_SSL_CERT_SUBJECT_STATE_OR_PROVINCE
PN_SSL_CERT_SUBJECT_CITY_OR_LOCALITY = _cproton.PN_SSL_CERT_SUBJECT_CITY_OR_LOCALITY
PN_SSL_CERT_SUBJECT_ORGANIZATION_NAME = _cproton.PN_SSL_CERT_SUBJECT_ORGANIZATION_NAME
PN_SSL_CERT_SUBJECT_ORGANIZATION_UNIT = _cproton.PN_SSL_CERT_SUBJECT_ORGANIZATION_UNIT
PN_SSL_CERT_SUBJECT_COMMON_NAME = _cproton.PN_SSL_CERT_SUBJECT_COMMON_NAME
PN_SSL_SHA1 = _cproton.PN_SSL_SHA1
PN_SSL_SHA256 = _cproton.PN_SSL_SHA256
PN_SSL_SHA512 = _cproton.PN_SSL_SHA512
PN_SSL_MD5 = _cproton.PN_SSL_MD5
PROTON_CODEC_H = _cproton.PROTON_CODEC_H
PN_NULL = _cproton.PN_NULL
PN_BOOL = _cproton.PN_BOOL
PN_UBYTE = _cproton.PN_UBYTE
PN_BYTE = _cproton.PN_BYTE
PN_USHORT = _cproton.PN_USHORT
PN_SHORT = _cproton.PN_SHORT
PN_UINT = _cproton.PN_UINT
PN_INT = _cproton.PN_INT
PN_CHAR = _cproton.PN_CHAR
PN_ULONG = _cproton.PN_ULONG
PN_LONG = _cproton.PN_LONG
PN_TIMESTAMP = _cproton.PN_TIMESTAMP
PN_FLOAT = _cproton.PN_FLOAT
PN_DOUBLE = _cproton.PN_DOUBLE
PN_DECIMAL32 = _cproton.PN_DECIMAL32
PN_DECIMAL64 = _cproton.PN_DECIMAL64
PN_DECIMAL128 = _cproton.PN_DECIMAL128
PN_UUID = _cproton.PN_UUID
PN_BINARY = _cproton.PN_BINARY
PN_STRING = _cproton.PN_STRING
PN_SYMBOL = _cproton.PN_SYMBOL
PN_DESCRIBED = _cproton.PN_DESCRIBED
PN_ARRAY = _cproton.PN_ARRAY
PN_LIST = _cproton.PN_LIST
PN_MAP = _cproton.PN_MAP
PN_INVALID = _cproton.PN_INVALID

def pn_type_name(type):
    return _cproton.pn_type_name(type)

def pn_data(capacity):
    return _cproton.pn_data(capacity)

def pn_data_free(data):
    return _cproton.pn_data_free(data)

def pn_data_errno(data):
    return _cproton.pn_data_errno(data)

def pn_data_error(data):
    return _cproton.pn_data_error(data)

def pn_data_fill(*args):
    return _cproton.pn_data_fill(*args)

def pn_data_scan(*args):
    return _cproton.pn_data_scan(*args)

def pn_data_clear(data):
    return _cproton.pn_data_clear(data)

def pn_data_size(data):
    return _cproton.pn_data_size(data)

def pn_data_rewind(data):
    return _cproton.pn_data_rewind(data)

def pn_data_next(data):
    return _cproton.pn_data_next(data)

def pn_data_prev(data):
    return _cproton.pn_data_prev(data)

def pn_data_enter(data):
    return _cproton.pn_data_enter(data)

def pn_data_exit(data):
    return _cproton.pn_data_exit(data)

def pn_data_lookup(data, name):
    return _cproton.pn_data_lookup(data, name)

def pn_data_type(data):
    return _cproton.pn_data_type(data)

def pn_data_print(data):
    return _cproton.pn_data_print(data)

def pn_data_encoded_size(data):
    return _cproton.pn_data_encoded_size(data)

def pn_data_put_list(data):
    return _cproton.pn_data_put_list(data)

def pn_data_put_map(data):
    return _cproton.pn_data_put_map(data)

def pn_data_put_array(data, described, type):
    return _cproton.pn_data_put_array(data, described, type)

def pn_data_put_described(data):
    return _cproton.pn_data_put_described(data)

def pn_data_put_null(data):
    return _cproton.pn_data_put_null(data)

def pn_data_put_bool(data, b):
    return _cproton.pn_data_put_bool(data, b)

def pn_data_put_ubyte(data, ub):
    return _cproton.pn_data_put_ubyte(data, ub)

def pn_data_put_byte(data, b):
    return _cproton.pn_data_put_byte(data, b)

def pn_data_put_ushort(data, us):
    return _cproton.pn_data_put_ushort(data, us)

def pn_data_put_short(data, s):
    return _cproton.pn_data_put_short(data, s)

def pn_data_put_uint(data, ui):
    return _cproton.pn_data_put_uint(data, ui)

def pn_data_put_int(data, i):
    return _cproton.pn_data_put_int(data, i)

def pn_data_put_char(data, c):
    return _cproton.pn_data_put_char(data, c)

def pn_data_put_ulong(data, ul):
    return _cproton.pn_data_put_ulong(data, ul)

def pn_data_put_long(data, l):
    return _cproton.pn_data_put_long(data, l)

def pn_data_put_timestamp(data, t):
    return _cproton.pn_data_put_timestamp(data, t)

def pn_data_put_float(data, f):
    return _cproton.pn_data_put_float(data, f)

def pn_data_put_double(data, d):
    return _cproton.pn_data_put_double(data, d)

def pn_data_put_decimal32(data, d):
    return _cproton.pn_data_put_decimal32(data, d)

def pn_data_put_decimal64(data, d):
    return _cproton.pn_data_put_decimal64(data, d)

def pn_data_put_decimal128(data, d):
    return _cproton.pn_data_put_decimal128(data, d)

def pn_data_put_uuid(data, u):
    return _cproton.pn_data_put_uuid(data, u)

def pn_data_put_binary(data, bytes):
    return _cproton.pn_data_put_binary(data, bytes)

def pn_data_put_string(data, string):
    return _cproton.pn_data_put_string(data, string)

def pn_data_put_symbol(data, symbol):
    return _cproton.pn_data_put_symbol(data, symbol)

def pn_data_get_list(data):
    return _cproton.pn_data_get_list(data)

def pn_data_get_map(data):
    return _cproton.pn_data_get_map(data)

def pn_data_get_array(data):
    return _cproton.pn_data_get_array(data)

def pn_data_is_array_described(data):
    return _cproton.pn_data_is_array_described(data)

def pn_data_get_array_type(data):
    return _cproton.pn_data_get_array_type(data)

def pn_data_is_described(data):
    return _cproton.pn_data_is_described(data)

def pn_data_is_null(data):
    return _cproton.pn_data_is_null(data)

def pn_data_get_bool(data):
    return _cproton.pn_data_get_bool(data)

def pn_data_get_ubyte(data):
    return _cproton.pn_data_get_ubyte(data)

def pn_data_get_byte(data):
    return _cproton.pn_data_get_byte(data)

def pn_data_get_ushort(data):
    return _cproton.pn_data_get_ushort(data)

def pn_data_get_short(data):
    return _cproton.pn_data_get_short(data)

def pn_data_get_uint(data):
    return _cproton.pn_data_get_uint(data)

def pn_data_get_int(data):
    return _cproton.pn_data_get_int(data)

def pn_data_get_char(data):
    return _cproton.pn_data_get_char(data)

def pn_data_get_ulong(data):
    return _cproton.pn_data_get_ulong(data)

def pn_data_get_long(data):
    return _cproton.pn_data_get_long(data)

def pn_data_get_timestamp(data):
    return _cproton.pn_data_get_timestamp(data)

def pn_data_get_float(data):
    return _cproton.pn_data_get_float(data)

def pn_data_get_double(data):
    return _cproton.pn_data_get_double(data)

def pn_data_get_decimal32(data):
    return _cproton.pn_data_get_decimal32(data)

def pn_data_get_decimal64(data):
    return _cproton.pn_data_get_decimal64(data)

def pn_data_get_decimal128(data):
    return _cproton.pn_data_get_decimal128(data)

def pn_data_get_uuid(data):
    return _cproton.pn_data_get_uuid(data)

def pn_data_get_binary(data):
    return _cproton.pn_data_get_binary(data)

def pn_data_get_string(data):
    return _cproton.pn_data_get_string(data)

def pn_data_get_symbol(data):
    return _cproton.pn_data_get_symbol(data)

def pn_data_get_bytes(data):
    return _cproton.pn_data_get_bytes(data)

def pn_data_copy(data, src):
    return _cproton.pn_data_copy(data, src)

def pn_data_append(data, src):
    return _cproton.pn_data_append(data, src)

def pn_data_appendn(data, src, limit):
    return _cproton.pn_data_appendn(data, src, limit)

def pn_data_narrow(data):
    return _cproton.pn_data_narrow(data)

def pn_data_widen(data):
    return _cproton.pn_data_widen(data)

def pn_data_point(data):
    return _cproton.pn_data_point(data)

def pn_data_restore(data, point):
    return _cproton.pn_data_restore(data, point)

def pn_data_dump(data):
    return _cproton.pn_data_dump(data)

def pn_cast_pn_connection(x):
    return _cproton.pn_cast_pn_connection(x)

def pn_cast_pn_session(x):
    return _cproton.pn_cast_pn_session(x)

def pn_cast_pn_link(x):
    return _cproton.pn_cast_pn_link(x)

def pn_cast_pn_delivery(x):
    return _cproton.pn_cast_pn_delivery(x)

def pn_cast_pn_transport(x):
    return _cproton.pn_cast_pn_transport(x)
PROTON_CONNECTION_DRIVER_H = _cproton.PROTON_CONNECTION_DRIVER_H

def pn_connection_driver_init(arg1, arg2, arg3):
    return _cproton.pn_connection_driver_init(arg1, arg2, arg3)

def pn_connection_driver_bind(d):
    return _cproton.pn_connection_driver_bind(d)

def pn_connection_driver_destroy(arg1):
    return _cproton.pn_connection_driver_destroy(arg1)

def pn_connection_driver_release_connection(d):
    return _cproton.pn_connection_driver_release_connection(d)

def pn_connection_driver_read_buffer_sized(arg1, n):
    return _cproton.pn_connection_driver_read_buffer_sized(arg1, n)

def pn_connection_driver_read_buffer(arg1):
    return _cproton.pn_connection_driver_read_buffer(arg1)

def pn_connection_driver_read_done(arg1, n):
    return _cproton.pn_connection_driver_read_done(arg1, n)

def pn_connection_driver_read_close(arg1):
    return _cproton.pn_connection_driver_read_close(arg1)

def pn_connection_driver_read_closed(arg1):
    return _cproton.pn_connection_driver_read_closed(arg1)

def pn_connection_driver_write_buffer(arg1):
    return _cproton.pn_connection_driver_write_buffer(arg1)

def pn_connection_driver_write_done(arg1, n):
    return _cproton.pn_connection_driver_write_done(arg1, n)

def pn_connection_driver_write_close(arg1):
    return _cproton.pn_connection_driver_write_close(arg1)

def pn_connection_driver_write_closed(arg1):
    return _cproton.pn_connection_driver_write_closed(arg1)

def pn_connection_driver_close(c):
    return _cproton.pn_connection_driver_close(c)

def pn_connection_driver_next_event(arg1):
    return _cproton.pn_connection_driver_next_event(arg1)

def pn_connection_driver_has_event(arg1):
    return _cproton.pn_connection_driver_has_event(arg1)

def pn_connection_driver_finished(arg1):
    return _cproton.pn_connection_driver_finished(arg1)

def pn_connection_driver_errorf(*args):
    return _cproton.pn_connection_driver_errorf(*args)

def pn_connection_driver_log(d, msg):
    return _cproton.pn_connection_driver_log(d, msg)

def pn_connection_driver_ptr(connection):
    return _cproton.pn_connection_driver_ptr(connection)
PROTON_CID_H = _cproton.PROTON_CID_H
CID_pn_object = _cproton.CID_pn_object
CID_pn_void = _cproton.CID_pn_void
CID_pn_weakref = _cproton.CID_pn_weakref
CID_pn_string = _cproton.CID_pn_string
CID_pn_list = _cproton.CID_pn_list
CID_pn_map = _cproton.CID_pn_map
CID_pn_hash = _cproton.CID_pn_hash
CID_pn_record = _cproton.CID_pn_record
CID_pn_collector = _cproton.CID_pn_collector
CID_pn_event = _cproton.CID_pn_event
CID_pn_buffer = _cproton.CID_pn_buffer
CID_pn_error = _cproton.CID_pn_error
CID_pn_data = _cproton.CID_pn_data
CID_pn_connection = _cproton.CID_pn_connection
CID_pn_session = _cproton.CID_pn_session
CID_pn_link = _cproton.CID_pn_link
CID_pn_delivery = _cproton.CID_pn_delivery
CID_pn_transport = _cproton.CID_pn_transport
CID_pn_message = _cproton.CID_pn_message
CID_pn_reactor = _cproton.CID_pn_reactor
CID_pn_handler = _cproton.CID_pn_handler
CID_pn_timer = _cproton.CID_pn_timer
CID_pn_task = _cproton.CID_pn_task
CID_pn_io = _cproton.CID_pn_io
CID_pn_selector = _cproton.CID_pn_selector
CID_pn_selectable = _cproton.CID_pn_selectable
CID_pn_url = _cproton.CID_pn_url
CID_pn_strdup = _cproton.CID_pn_strdup
CID_pn_listener = _cproton.CID_pn_listener
CID_pn_proactor = _cproton.CID_pn_proactor
CID_pn_listener_socket = _cproton.CID_pn_listener_socket
CID_pn_raw_connection = _cproton.CID_pn_raw_connection

cvar = _cproton.cvar
PN_PYREF = cvar.PN_PYREF
pn_bytes_null = cvar.pn_bytes_null
pn_rwbytes_null = cvar.pn_rwbytes_null
PN_OBJECT = cvar.PN_OBJECT
PN_VOID = cvar.PN_VOID




#!/usr/bin/env python3
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

from proton.reactor import Container, Handler


class Recurring(Handler):
    def __init__(self, period):
        self.period = period

    def on_reactor_init(self, event):
        self.container = event.reactor
        self.container.schedule(self.period, self)

    def on_timer_task(self, event):
        print("Tick...")
        self.container.schedule(self.period, self)


try:
    container = Container(Recurring(1.0))
    container.run()
except KeyboardInterrupt:
    container.stop()
    print()



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

import socket

from urllib.parse import urlparse, urlunparse, quote, unquote


class Url(object):
    """
    **DEPRECATED** Simple URL parser/constructor.

    .. deprecated:: 0.27
        Use a ``str`` containing the URL instead.

    Handles URLs of the form:

        ``<scheme>://<user>:<password>@<host>:<port>/<path>``

    All components can be ``None`` if not specified in the URL string.

    The port can be specified as a service name, e.g. 'amqp' in the
    URL string but :class:`Url.Port` always gives the integer value.

    .. warning:: The placement of user and password in URLs is not
        recommended.  It can result in credentials leaking out in program
        logs.  Use connection configuration attributes instead.

    :ivar scheme: Url scheme e.g. 'amqp' or 'amqps'
    :ivar username: Username
    :ivar ~.password: Password
    :ivar ~.host: Host name, ipv6 literal or ipv4 dotted quad.
    :ivar ~.port: Integer port.
    :ivar host_port: Returns host:port

    :param url: URL string to parse.
    :type url: ``str``
    :param defaults: If ``True``, fill in missing default values in the URL.
        If ``False``, you can fill them in later by calling self.defaults()
    :type defaults: ``bool``
    :param kwargs: scheme, user, password, host, port, path.
        If specified, replaces corresponding part in url string.
    """

    AMQPS = "amqps"
    """URL scheme for the AMQP protocol secured with SSL."""

    AMQP = "amqp"
    """URL scheme for the AMQP protocol."""

    class Port(int):
        """An integer port number that can be constructed from a service name string"""

        def __new__(cls, value):
            """
            :param value: integer port number or string service name.
            """
            port = super(Url.Port, cls).__new__(cls, cls._port_int(value))
            setattr(port, 'name', str(value))
            return port

        def __eq__(self, x):
            return str(self) == x or int(self) == x

        def __ne__(self, x):
            return not self == x

        def __str__(self):
            return str(self.name)

        @staticmethod
        def _port_int(value):
            """Convert service, an integer or a service name, into an integer port number."""
            try:
                return int(value)
            except ValueError:
                try:
                    return socket.getservbyname(value)
                except socket.error:
                    # Not every system has amqp/amqps defined as a service
                    if value == Url.AMQPS:
                        return 5671
                    elif value == Url.AMQP:
                        return 5672
                    else:
                        raise ValueError("Not a valid port number or service name: '%s'" % value)

    def __init__(self, url=None, defaults=True, **kwargs):
        if isinstance(url, Url):
            self.scheme = url.scheme
            self.username = url.username
            self.password = url.password
            self._host = url._host
            self._port = url._port
            self._path = url._path
            self._params = url._params
            self._query = url._query
            self._fragment = url._fragment
        elif url:
            if not url.startswith('//'):
                p = url.partition(':')
                if '/' in p[0] or not p[2].startswith('//'):
                    url = '//' + url
            u = urlparse(url)
            if not u:
                raise ValueError("Invalid URL '%s'" % url)
            self.scheme = None if not u.scheme else u.scheme
            self.username = u.username and unquote(u.username)
            self.password = u.password and unquote(u.password)
            (self._host, self._port) = self._parse_host_port(u.netloc)
            self._path = None if not u.path else u.path
            self._params = u.params
            self._query = u.query
            self._fragment = u.fragment
        else:
            self.scheme = None
            self.username = None
            self.password = None
            self._host = None
            self._port = None
            self._path = None
            self._params = None
            self._query = None
            self._fragment = None
        for k in kwargs:  # Let kwargs override values parsed from url
            getattr(self, k)  # Check for invalid kwargs
            setattr(self, k, kwargs[k])
        if defaults:
            self.defaults()

    @staticmethod
    def _parse_host_port(nl):
        hostport = nl.split('@')[-1]
        hostportsplit = hostport.split(']')
        beforebrace = hostportsplit[0]
        afterbrace = hostportsplit[-1]

        if len(hostportsplit) == 1:
            beforebrace = ''
        else:
            beforebrace += ']'
        if ':' in afterbrace:
            afterbracesplit = afterbrace.split(':')
            port = afterbracesplit[1]
            host = (beforebrace + afterbracesplit[0]).lower()
            if not port:
                port = None
        else:
            host = (beforebrace + afterbrace).lower()
            port = None
        if not host:
            host = None
        return host, port

    @property
    def path(self):
        """
        The path segment of a URL

        :type: ``str``
        """
        return self._path if not self._path or self._path[0] != '/' else self._path[1:]

    @path.setter
    def path(self, p):
        self._path = p if p[0] == '/' else '/' + p

    @staticmethod
    def _ipv6literal(s):
        return s.startswith('[') and s.endswith(']')

    @property
    def host(self):
        """
        The host segment of a URL

        :type: ``str``
        """
        if self._host and self._ipv6literal(self._host):
            return self._host[1:-1]
        else:
            return self._host

    @host.setter
    def host(self, h):
        if ':' in h and not self._ipv6literal(h):
            self._host = '[' + h + ']'
        else:
            self._host = h

    @property
    def port(self):
        """
        The port number segment of a URL.

        :type: :class:`Url.Port`
        """
        return self._port and Url.Port(self._port)

    @port.setter
    def port(self, p):
        self._port = p

    @property
    def _netloc(self):
        hostport = ''
        if self._host:
            hostport = self._host
        if self._port:
            hostport += ':'
            hostport += str(self._port)
        userpart = ''
        if self.username:
            userpart += quote(self.username)
        if self.password:
            userpart += ':'
            userpart += quote(self.password)
        if self.username or self.password:
            userpart += '@'
        return userpart + hostport

    def __str__(self):
        if self.scheme \
                and not self._netloc and not self._path \
                and not self._params and not self._query and not self._fragment:
            return self.scheme + '://'
        return urlunparse((self.scheme or '', self._netloc or '', self._path or '',
                           self._params or '', self._query or '', self._fragment or ''))

    def __repr__(self):
        return "Url('%s')" % self

    def __eq__(self, x):
        return str(self) == str(x)

    def __ne__(self, x):
        return not self == x

    def defaults(self):
        """
        Fill in missing values (scheme, host or port) with defaults
        :return: self
        """
        self.scheme = self.scheme or self.AMQP
        self._host = self._host or '0.0.0.0'
        self._port = self._port or self.Port(self.scheme)
        return self



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

from proton import Message
from proton.reactor import Container
from proton.handlers import MessagingHandler


class Server(MessagingHandler):
    def __init__(self, host, address):
        super(Server, self).__init__()
        self.container = Container(self)
        self.conn = self.container.connect(host)
        self.receiver = self.container.create_receiver(self.conn, address)
        self.sender = self.container.create_sender(self.conn, None)

    def on_message(self, event):
        self.on_request(event.message.body, event.message.reply_to)

    def on_connection_close(self, endpoint, error):
        if error:
            print("Closed due to %s" % error)
        self.conn.close()

    def run(self):
        self.container.run()

    def send(self, response, reply_to):
        msg = Message(body=response)
        if self.sender:
            msg.address = reply_to
        self.sender.send(msg)

    def on_request(self, request, reply_to):
        pass



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

import threading

from cproton import PN_CONNECTION_BOUND, PN_CONNECTION_FINAL, PN_CONNECTION_INIT, PN_CONNECTION_LOCAL_CLOSE, \
    PN_CONNECTION_LOCAL_OPEN, PN_CONNECTION_REMOTE_CLOSE, PN_CONNECTION_REMOTE_OPEN, PN_CONNECTION_UNBOUND, PN_DELIVERY, \
    PN_LINK_FINAL, PN_LINK_FLOW, PN_LINK_INIT, PN_LINK_LOCAL_CLOSE, PN_LINK_LOCAL_DETACH, PN_LINK_LOCAL_OPEN, \
    PN_LINK_REMOTE_CLOSE, PN_LINK_REMOTE_DETACH, PN_LINK_REMOTE_OPEN, PN_PYREF, PN_SESSION_FINAL, PN_SESSION_INIT, \
    PN_SESSION_LOCAL_CLOSE, PN_SESSION_LOCAL_OPEN, PN_SESSION_REMOTE_CLOSE, PN_SESSION_REMOTE_OPEN, PN_TIMER_TASK, \
    PN_TRANSPORT, PN_TRANSPORT_CLOSED, PN_TRANSPORT_ERROR, PN_TRANSPORT_HEAD_CLOSED, PN_TRANSPORT_TAIL_CLOSED, \
    pn_cast_pn_connection, pn_cast_pn_delivery, pn_cast_pn_link, pn_cast_pn_session, pn_cast_pn_transport, \
    pn_class_name, pn_collector, pn_collector_free, pn_collector_more, pn_collector_peek, pn_collector_pop, \
    pn_collector_put, pn_collector_release, pn_event_class, pn_event_connection, pn_event_context, pn_event_delivery, \
    pn_event_link, pn_event_session, pn_event_transport, pn_event_type, pn_event_type_name, pn_py2void, pn_void2py

from ._delivery import Delivery
from ._endpoints import Connection, Link, Session
from ._transport import Transport
from typing import Any, List, Optional, Union, TYPE_CHECKING, Callable, Tuple, Type

if TYPE_CHECKING:
    from ._reactor import Container
    from ._endpoints import Receiver, Sender
    from ._handlers import ConnectSelectable
    from ._selectable import Selectable
    from types import TracebackType


class Collector:

    def __init__(self) -> None:
        self._impl = pn_collector()

    def put(self, obj: 'Selectable', etype: 'EventType') -> None:
        pn_collector_put(self._impl, PN_PYREF, pn_py2void(obj), etype.number)

    def peek(self) -> Optional['Event']:
        return Event.wrap(pn_collector_peek(self._impl))

    def more(self) -> bool:
        return pn_collector_more(self._impl)

    def pop(self) -> None:
        ev = self.peek()
        pn_collector_pop(self._impl)

    def release(self) -> None:
        pn_collector_release(self._impl)

    def __del__(self) -> None:
        pn_collector_free(self._impl)
        del self._impl


if "TypeExtender" not in globals():
    class TypeExtender:
        def __init__(self, number: int) -> None:
            self.number = number

        def next(self) -> int:
            try:
                return self.number
            finally:
                self.number += 1


class EventType(object):
    """
    Connects an event number to an event name, and is used
    internally by :class:`Event` to represent all known
    event types. A global list of events is maintained. An
    :class:`EventType` created with a name but no number is
    treated as an *extended* event, and is assigned an
    internal event number starting at 10000.
    """
    _lock = threading.Lock()
    _extended = TypeExtender(10000)
    TYPES = {}

    def __init__(self, name: Optional[str] = None, number: Optional[int] = None, method: Optional[str] = None) -> None:
        if name is None and number is None:
            raise TypeError("extended events require a name")
        try:
            self._lock.acquire()
            if name is None:
                name = pn_event_type_name(number)

            if number is None:
                number = self._extended.next()

            if method is None:
                method = "on_%s" % name

            self.name = name
            self.number = number
            self.method = method

            self.TYPES[number] = self
        finally:
            self._lock.release()

    def __repr__(self):
        return "EventType(name=%s, number=%d)" % (self.name, self.number)

    def __str__(self):
        return self.name


def _dispatch(handler: Any, method: str, *args) -> None:
    m = getattr(handler, method, None)
    if m:
        m(*args)
    elif hasattr(handler, "on_unhandled"):
        handler.on_unhandled(method, *args)


class EventBase(object):

    def __init__(self, type: EventType) -> None:
        self._type = type

    @property
    def type(self) -> EventType:
        """The type of this event."""
        return self._type

    @property
    def handler(self) -> Optional['Handler']:
        """The handler for this event type. Not implemented, always returns ``None``."""
        return None

    def dispatch(self, handler: 'Handler', type: Optional[EventType] = None) -> None:
        """
        Process this event by sending it to all known handlers that
        are valid for this event type.

        :param handler: Parent handler to process this event
        :param type: Event type
        """
        type = type or self._type
        _dispatch(handler, type.method, self)
        if hasattr(handler, "handlers"):
            for h in handler.handlers:
                self.dispatch(h, type)

    def __repr__(self) -> str:
        return "%s(%r)" % (self._type, self.context)


def _core(number: int, method: str) -> EventType:
    return EventType(number=number, method=method)


def _internal(name: str) -> EventType:
    return EventType(name=name)


wrappers = {
    "pn_void": lambda x: pn_void2py(x),
    "pn_pyref": lambda x: pn_void2py(x),
    "pn_connection": lambda x: Connection.wrap(pn_cast_pn_connection(x)),
    "pn_session": lambda x: Session.wrap(pn_cast_pn_session(x)),
    "pn_link": lambda x: Link.wrap(pn_cast_pn_link(x)),
    "pn_delivery": lambda x: Delivery.wrap(pn_cast_pn_delivery(x)),
    "pn_transport": lambda x: Transport.wrap(pn_cast_pn_transport(x))
}


class Event(EventBase):
    """
    Notification of a state change in the protocol engine.
    """
    TIMER_TASK = _core(PN_TIMER_TASK, "on_timer_task")
    """A timer event has occurred."""

    CONNECTION_INIT = _core(PN_CONNECTION_INIT, "on_connection_init")
    """
    The connection has been created. This is the first event that
    will ever be issued for a connection. Events of this type point
    to the relevant connection.
    """

    CONNECTION_BOUND = _core(PN_CONNECTION_BOUND, "on_connection_bound")
    """
    The connection has been bound to a transport. This event is
    issued when the :meth:`Transport.bind` operation is invoked.
    """

    CONNECTION_UNBOUND = _core(PN_CONNECTION_UNBOUND, "on_connection_unbound")
    """
    The connection has been unbound from its transport. This event is
    issued when the :meth:`Transport.unbind` operation is invoked.
    """

    CONNECTION_LOCAL_OPEN = _core(PN_CONNECTION_LOCAL_OPEN, "on_connection_local_open")
    """
    The local connection endpoint has been closed. Events of this
    type point to the relevant connection.
    """

    CONNECTION_LOCAL_CLOSE = _core(PN_CONNECTION_LOCAL_CLOSE, "on_connection_local_close")
    """
    The local connection endpoint has been closed. Events of this
    type point to the relevant connection.
    """

    CONNECTION_REMOTE_OPEN = _core(PN_CONNECTION_REMOTE_OPEN, "on_connection_remote_open")
    """
    The remote endpoint has opened the connection. Events of this
    type point to the relevant connection.
    """

    CONNECTION_REMOTE_CLOSE = _core(PN_CONNECTION_REMOTE_CLOSE, "on_connection_remote_close")
    """
    The remote endpoint has closed the connection. Events of this
    type point to the relevant connection.
    """

    CONNECTION_FINAL = _core(PN_CONNECTION_FINAL, "on_connection_final")
    """
    The connection has been freed and any outstanding processing has
    been completed. This is the final event that will ever be issued
    for a connection.
    """

    SESSION_INIT = _core(PN_SESSION_INIT, "on_session_init")
    """
    The session has been created. This is the first event that will
    ever be issued for a session.
    """

    SESSION_LOCAL_OPEN = _core(PN_SESSION_LOCAL_OPEN, "on_session_local_open")
    """
    The local session endpoint has been opened. Events of this type
    point to the relevant session.
    """

    SESSION_LOCAL_CLOSE = _core(PN_SESSION_LOCAL_CLOSE, "on_session_local_close")
    """
    The local session endpoint has been closed. Events of this type
    point ot the relevant session.
    """

    SESSION_REMOTE_OPEN = _core(PN_SESSION_REMOTE_OPEN, "on_session_remote_open")
    """
    The remote endpoint has opened the session. Events of this type
    point to the relevant session.
    """

    SESSION_REMOTE_CLOSE = _core(PN_SESSION_REMOTE_CLOSE, "on_session_remote_close")
    """
    The remote endpoint has closed the session. Events of this type
    point to the relevant session.
    """

    SESSION_FINAL = _core(PN_SESSION_FINAL, "on_session_final")
    """
    The session has been freed and any outstanding processing has
    been completed. This is the final event that will ever be issued
    for a session.
    """

    LINK_INIT = _core(PN_LINK_INIT, "on_link_init")
    """
    The link has been created. This is the first event that will ever
    be issued for a link.
    """

    LINK_LOCAL_OPEN = _core(PN_LINK_LOCAL_OPEN, "on_link_local_open")
    """
    The local link endpoint has been opened. Events of this type
    point ot the relevant link.
    """

    LINK_LOCAL_CLOSE = _core(PN_LINK_LOCAL_CLOSE, "on_link_local_close")
    """
    The local link endpoint has been closed. Events of this type
    point to the relevant link.
    """

    LINK_LOCAL_DETACH = _core(PN_LINK_LOCAL_DETACH, "on_link_local_detach")
    """
    The local link endpoint has been detached. Events of this type
    point to the relevant link.
    """

    LINK_REMOTE_OPEN = _core(PN_LINK_REMOTE_OPEN, "on_link_remote_open")
    """
    The remote endpoint has opened the link. Events of this type
    point to the relevant link.
    """

    LINK_REMOTE_CLOSE = _core(PN_LINK_REMOTE_CLOSE, "on_link_remote_close")
    """
    The remote endpoint has closed the link. Events of this type
    point to the relevant link.
    """

    LINK_REMOTE_DETACH = _core(PN_LINK_REMOTE_DETACH, "on_link_remote_detach")
    """
    The remote endpoint has detached the link. Events of this type
    point to the relevant link.
    """

    LINK_FLOW = _core(PN_LINK_FLOW, "on_link_flow")
    """
    The flow control state for a link has changed. Events of this
    type point to the relevant link.
    """

    LINK_FINAL = _core(PN_LINK_FINAL, "on_link_final")
    """
    The link has been freed and any outstanding processing has been
    completed. This is the final event that will ever be issued for a
    link. Events of this type point to the relevant link.
    """

    DELIVERY = _core(PN_DELIVERY, "on_delivery")
    """
    A delivery has been created or updated. Events of this type point
    to the relevant delivery.
    """

    TRANSPORT = _core(PN_TRANSPORT, "on_transport")
    """
    The transport has new data to read and/or write. Events of this
    type point to the relevant transport.
    """

    TRANSPORT_ERROR = _core(PN_TRANSPORT_ERROR, "on_transport_error")
    """
    Indicates that a transport error has occurred. Use :attr:`Transport.condition`
    to access the details of the error from the associated transport.
    """

    TRANSPORT_HEAD_CLOSED = _core(PN_TRANSPORT_HEAD_CLOSED, "on_transport_head_closed")
    """
    Indicates that the "head" or writing end of the transport has been closed. This
    means the transport will never produce more bytes for output to
    the network. Events of this type point to the relevant transport.
    """

    TRANSPORT_TAIL_CLOSED = _core(PN_TRANSPORT_TAIL_CLOSED, "on_transport_tail_closed")
    """
    Indicates that the "tail" of the transport has been closed. This
    means the transport will never be able to process more bytes from
    the network. Events of this type point to the relevant transport.
    """

    TRANSPORT_CLOSED = _core(PN_TRANSPORT_CLOSED, "on_transport_closed")
    """
    Indicates that the both the "head" and "tail" of the transport are
    closed. Events of this type point to the relevant transport.
    """

    # These events are now internal events in the python code
    REACTOR_INIT = _internal("reactor_init")
    """
    A reactor has been started. Events of this type point to the
    reactor.
    """

    REACTOR_QUIESCED = _internal("reactor_quiesced")
    """
    A reactor has no more events to process. Events of this type
    point to the reactor.
    """

    REACTOR_FINAL = _internal("reactor_final")
    """
    A reactor has been stopped. Events of this type point to the
    reactor.
    """

    SELECTABLE_INIT = _internal("selectable_init")
    SELECTABLE_UPDATED = _internal("selectable_updated")
    SELECTABLE_READABLE = _internal("selectable_readable")
    SELECTABLE_WRITABLE = _internal("selectable_writable")
    SELECTABLE_EXPIRED = _internal("selectable_expired")
    SELECTABLE_ERROR = _internal("selectable_error")
    SELECTABLE_FINAL = _internal("selectable_final")

    @staticmethod
    def wrap(impl):
        if impl is None:
            return None

        number = pn_event_type(impl)
        cls = pn_event_class(impl)

        if cls:
            clsname = pn_class_name(cls)
            context = wrappers[clsname](pn_event_context(impl))

            # check for an application defined ApplicationEvent and return that.  This
            # avoids an expensive wrap operation invoked by event.context
            if cls == PN_PYREF and isinstance(context, EventBase):
                return context
        else:
            clsname = None

        event = Event(impl, number, clsname, context)
        return event

    def __init__(self, impl, number, clsname, context):
        self._type = EventType.TYPES[number]
        self._clsname = clsname
        self._context = context

        # Do all this messing around to avoid duplicate wrappers
        if issubclass(type(context), Delivery):
            self._delivery = context
        else:
            self._delivery = Delivery.wrap(pn_event_delivery(impl))
        if self._delivery:
            self._link = self._delivery.link
        elif issubclass(type(context), Link):
            self._link = context
        else:
            self._link = Link.wrap(pn_event_link(impl))
        if self._link:
            self._session = self._link.session
        elif issubclass(type(context), Session):
            self._session = context
        else:
            self._session = Session.wrap(pn_event_session(impl))
        if self._session:
            self._connection = self._session.connection
        elif issubclass(type(context), Connection):
            self._connection = context
        else:
            self._connection = Connection.wrap(pn_event_connection(impl))

        if issubclass(type(context), Transport):
            self._transport = context
        else:
            self._transport = Transport.wrap(pn_event_transport(impl))

    @property
    def clazz(self) -> str:
        """
        The name of the class associated with the event context.
        """
        return self._clsname

    @property
    def context(self) -> Union[Optional[Any], Connection, Session, Link, Delivery, Transport]:
        """
        The context object associated with the event.

        :type: Depends on the type of event, and include the following:
               - :class:`Connection`
               - :class:`Session`
               - :class:`Link`
               - :class:`Delivery`
               - :class:`Transport`
        """
        return self._context

    @property
    def handler(self) -> Optional['Handler']:
        """
        The handler for this event. The handler is determined by looking
        at the following in order:

        - The link
        - The session
        - The connection
        - The context object with an attribute "handler"

        If none of these has a handler, then ``None`` is returned.
        """
        l = self.link
        if l:
            h = l.handler
            if h:
                return h
        s = self.session
        if s:
            h = s.handler
            if h:
                return h
        c = self.connection
        if c:
            h = c.handler
            if h:
                return h
        c = self.context
        if not c or not hasattr(c, 'handler'):
            return None
        h = c.handler
        return h

    @property
    def reactor(self) -> 'Container':
        """
        **Deprecated** - The :class:`reactor.Container` (was reactor) associated with the event.
        """
        return self.container

    @property
    def container(self) -> 'Container':
        """
        The :class:`reactor.Container` associated with the event.
        """
        return self._transport._reactor

    def __getattr__(self, name: str) -> Any:
        """
        This will look for a property of the event as an attached context object of the same
        type as the property (but lowercase)
        """
        c = self.context
        # Direct type or subclass of type
        if type(c).__name__.lower() == name or name in [x.__name__.lower() for x in type(c).__bases__]:
            return c

        # If the attached object is the wrong type then see if *it* has a property of that name
        return getattr(c, name, None)

    @property
    def transport(self) -> Optional[Transport]:
        """
        The transport associated with the event, or ``None`` if none
        is associated with it.
        """
        return self._transport

    @property
    def connection(self) -> Optional[Connection]:
        """
        The connection associated with the event, or ``None`` if none
        is associated with it.
        """
        return self._connection

    @property
    def session(self) -> Optional[Session]:
        """
        The session associated with the event, or ``None`` if none
        is associated with it.
        """
        return self._session

    @property
    def link(self) -> Optional[Union['Receiver', 'Sender']]:
        """
        The link associated with the event, or ``None`` if none
        is associated with it.
        """
        return self._link

    @property
    def sender(self) -> Optional['Sender']:
        """
        The sender link associated with the event, or ``None`` if
        none is associated with it. This is essentially an alias for
        ``link`` property, that does an additional check on the type of the
        link.
        """
        l = self.link
        if l and l.is_sender:
            return l
        else:
            return None

    @property
    def receiver(self) -> Optional['Receiver']:
        """
        The receiver link associated with the event, or ``None`` if
        none is associated with it. This is essentially an alias for
        ``link`` property, that does an additional check on the type of the link.
        """
        l = self.link
        if l and l.is_receiver:
            return l
        else:
            return None

    @property
    def delivery(self) -> Optional[Delivery]:
        """
        The delivery associated with the event, or ``None`` if none
        is associated with it.
        """
        return self._delivery


class LazyHandlers(object):
    def __get__(self, obj: 'Handler', clazz: Any) -> Union['LazyHandlers', List[Any]]:
        if obj is None:
            return self
        ret = []
        obj.__dict__['handlers'] = ret
        return ret


class Handler(object):
    """
    An abstract handler for events which supports child handlers.
    """
    handlers = LazyHandlers()

    # TODO What to do with on_error?
    def add(
            self,
            handler: Any,
            on_error: Optional[Callable[[Tuple[Type[BaseException], BaseException, 'TracebackType']], None]] = None,
    ) -> None:
        """
        Add a child handler

        :param handler: A child handler
        :type handler: :class:`Handler` or one of its derivatives.
        :param on_error: Not used
        """
        self.handlers.append(handler)

    def on_unhandled(self, method: str, *args) -> None:
        """
        The callback for handling events which are not handled by
        any other handler.

        :param method: The name of the intended handler method.
        :param args: Arguments for the intended handler method.
        """
        pass



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

from typing import Optional, Union, TYPE_CHECKING, Any

from ._events import Event
from ._io import PN_INVALID_SOCKET

if TYPE_CHECKING:
    from ._events import EventType
    from ._reactor import Container, EventInjector
    from socket import socket


class Selectable(object):

    def __init__(
            self,
            delegate: Optional[Union['EventInjector', 'socket']],
            reactor: 'Container',
    ) -> None:
        self._delegate = delegate
        self.reading = False
        self.writing = False
        self._deadline = 0
        self._terminal = False
        self._released = False
        self._terminated = False
        self._reactor = reactor
        self.push_event(self, Event.SELECTABLE_INIT)

    def close(self) -> None:
        if self._delegate and not self._released:
            self._delegate.close()

    def fileno(self) -> int:
        if self._delegate:
            return self._delegate.fileno()
        else:
            return PN_INVALID_SOCKET

    def __getattr__(self, name: str) -> Any:
        return getattr(self._delegate, name)

    @property
    def deadline(self) -> Optional[float]:
        tstamp = self._deadline
        if tstamp:
            return tstamp
        else:
            return None

    @deadline.setter
    def deadline(self, deadline: Optional[float]) -> None:
        if not deadline:
            self._deadline = 0
        else:
            self._deadline = deadline

    def push_event(
            self,
            context: 'Selectable',
            etype: 'EventType',
    ) -> None:
        self._reactor.push_event(context, etype)

    def update(self) -> None:
        if not self._terminated:
            if self._terminal:
                self._terminated = True
                self.push_event(self, Event.SELECTABLE_FINAL)
            else:
                self.push_event(self, Event.SELECTABLE_UPDATED)

    def readable(self) -> None:
        self.push_event(self, Event.SELECTABLE_READABLE)

    def writable(self) -> None:
        self.push_event(self, Event.SELECTABLE_WRITABLE)

    def expired(self) -> None:
        self.push_event(self, Event.SELECTABLE_EXPIRED)

    @property
    def is_terminal(self) -> bool:
        return self._terminal

    def terminate(self) -> None:
        self._terminal = True

    def release(self) -> None:
        self._released = True



#!/usr/bin/env python3
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

from proton import Message
from proton.reactor import Container
from proton.handlers import MessagingHandler, TransactionHandler


class TxRequest(TransactionHandler):
    def __init__(self, response, sender, request_delivery):
        super(TxRequest, self).__init__()
        self.response = response
        self.sender = sender
        self.request_delivery = request_delivery

    def on_transaction_declared(self, event):
        event.transaction.send(self.sender, self.response)
        event.transaction.accept(self.request_delivery)
        event.transaction.commit()

    def on_transaction_committed(self, event):
        print("Request processed successfully")

    def on_transaction_aborted(self, event):
        print("Request processing aborted")


class TxServer(MessagingHandler):
    def __init__(self, host, address):
        super(TxServer, self).__init__(auto_accept=False)
        self.host = host
        self.address = address

    def on_start(self, event):
        self.container = event.container
        self.conn = event.container.connect(self.host, reconnect=False)
        self.receiver = event.container.create_receiver(self.conn, self.address)
        self.senders = {}
        self.relay = None

    def on_message(self, event):
        sender = self.relay
        if not sender:
            sender = self.senders.get(event.message.reply_to)
        if not sender:
            sender = self.container.create_sender(self.conn, event.message.reply_to)
            self.senders[event.message.reply_to] = sender

        response = Message(address=event.message.reply_to, body=event.message.body.upper(),
                           correlation_id=event.message.correlation_id)
        self.container.declare_transaction(self.conn, handler=TxRequest(response, sender, event.delivery))

    def on_connection_open(self, event):
        if event.connection.remote_offered_capabilities and 'ANONYMOUS-RELAY' in event.connection.remote_offered_capabilities:
            self.relay = self.container.create_sender(self.conn, None)


try:
    Container(TxServer("localhost:5672", "examples")).run()
except KeyboardInterrupt:
    pass



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

from ._reactor import Container, ApplicationEvent, EventInjector, Handler,\
    LinkOption, ReceiverOption, SenderOption,\
    AtLeastOnce, AtMostOnce, DynamicNodeProperties, Filter, Selector, \
    DurableSubscription, Copy, Move, Backoff, Transaction

__all__ = [
    'Container',
    'ApplicationEvent',
    'EventInjector',
    'Handler',
    'LinkOption',
    'ReceiverOption',
    'SenderOption',
    'AtLeastOnce',
    'AtMostOnce',
    'DynamicNodeProperties',
    'Filter',
    'Selector',
    'DurableSubscription',
    'Copy',
    'Move',
    'Backoff',
    'Transaction'
]



#!/usr/bin/env python3
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

from proton import Message
from proton.handlers import MessagingHandler
from proton.reactor import Container


class HelloWorld(MessagingHandler):
    def __init__(self, server, address):
        super(HelloWorld, self).__init__()
        self.server = server
        self.address = address

    def on_start(self, event):
        conn = event.container.connect(self.server)
        event.container.create_receiver(conn, self.address)
        event.container.create_sender(conn, self.address)

    def on_sendable(self, event):
        event.sender.send(Message(body="Hello World!"))
        event.sender.close()

    def on_message(self, event):
        print(event.message.body)
        event.connection.close()


Container(HelloWorld("localhost:5672", "examples")).run()



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

from typing import Optional, TYPE_CHECKING

from cproton import pn_condition_clear, pn_condition_set_name, pn_condition_set_description, pn_condition_info, \
    pn_condition_is_set, pn_condition_get_name, pn_condition_get_description

from ._data import Data, dat2obj

if TYPE_CHECKING:
    from ._data import PythonAMQPData


class Condition:
    """
    An AMQP Condition object. Conditions hold exception information
    pertaining to the closing of an AMQP endpoint such as a :class:`Connection`,
    :class:`Session`, or :class:`Link`. Conditions also hold similar information
    pertaining to deliveries that have reached terminal states.
    Connections, Sessions, Links, and Deliveries may all have local and
    remote conditions associated with them.

    The local condition may be modified by the local endpoint to signal
    a particular condition to the remote peer. The remote condition may
    be examined by the local endpoint to detect whatever condition the
    remote peer may be signaling. Although often conditions are used to
    indicate errors, not all conditions are errors *per/se*, e.g.
    conditions may be used to redirect a connection from one host to
    another.

    Every condition has a short symbolic name, a longer description,
    and an additional info map associated with it. The name identifies
    the formally defined condition, and the map contains additional
    information relevant to the identified condition.

    :ivar ~.name: The name of the condition.
    :ivar ~.description: A description of the condition.
    :ivar ~.info: A data object that holds the additional information associated
        with the condition. The data object may be used both to access and to
        modify the additional information associated with the condition.
    """

    def __init__(
            self,
            name: str,
            description: Optional[str] = None,
            info: Optional['PythonAMQPData'] = None
    ) -> None:
        self.name = name
        self.description = description
        self.info = info

    def __repr__(self) -> str:
        return "Condition(%s)" % ", ".join([repr(x) for x in
                                            (self.name, self.description, self.info)
                                            if x])

    def __eq__(self, o: 'Condition') -> bool:
        if not isinstance(o, Condition):
            return False
        return self.name == o.name and \
            self.description == o.description and \
            self.info == o.info


def obj2cond(obj, cond: Condition) -> None:
    pn_condition_clear(cond)
    if obj:
        pn_condition_set_name(cond, str(obj.name))
        pn_condition_set_description(cond, obj.description)
        info = Data(pn_condition_info(cond))
        if obj.info:
            info.put_object(obj.info)


def cond2obj(cond) -> Optional[Condition]:
    if pn_condition_is_set(cond):
        return Condition(pn_condition_get_name(cond),
                         pn_condition_get_description(cond),
                         dat2obj(pn_condition_info(cond)))
    else:
        return None



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License
#

# Extends the subprocess module to use runtime checkers, and report stderr output.

import os
import re
import subprocess
import tempfile
from subprocess import PIPE


def in_path(name):
    """Look for name in the PATH"""
    for path in os.environ["PATH"].split(os.pathsep):
        f = os.path.join(path, name)
        if os.path.isfile(f) and os.access(f, os.X_OK):
            return f


class TestProcessError(Exception):
    def __init__(self, proc, what, output=None):
        self.output = output
        sep = "\n%s stderr(%s) %s\n" % ("_" * 32, proc.pid, "_" * 32)
        error = sep + proc.error + sep if proc.error else ""
        super(TestProcessError, self).__init__("%s pid=%s exit=%s: %s%s" % (
            proc.cmd, proc.pid, proc.returncode, what, error))


class Popen(subprocess.Popen):
    """
    Add TEST_EXE_PREFIX to the command, check stderr for runtime checker output.
    In a 'with' statement it runs check_wait() on exit from the block, or
    check_kill() if initialized with kill_me=True
    """

    def __init__(self, *args, **kwargs):
        """
        Takes all args and kwargs of subprocess.Popen except stdout, stderr, universal_newlines
        kill_me=True runs check_kill() in __exit__() instead of check_wait()
        """
        self.on_exit = self.check_kill if kwargs.pop('kill_me', False) else self.check_wait
        self.errfile = tempfile.NamedTemporaryFile(delete=False)
        kwargs.update({'universal_newlines': True, 'stdout': PIPE, 'stderr': self.errfile})
        prefix = os.environ.get("TEST_EXE_PREFIX")
        if prefix:
            args = [prefix.split() + args[0]] + list(args[1:])
        self.cmd = args[0]
        super(Popen, self).__init__(*args, **kwargs)

    def check_wait(self):
        if self.wait() or self.error:
            raise TestProcessError(self, "check_wait")

    def communicate(self, *args, **kwargs):
        result = super(Popen, self).communicate(*args, **kwargs)
        if self.returncode or self.error:
            raise TestProcessError(self, "check_communicate", result[0])
        return result

    def check_kill(self):
        """Raise if process has already exited, kill and raise if self.error is not empty"""
        if self.poll() is None:
            self.kill()
            self.wait()
            self.stdout.close()     # Doesn't get closed if killed
            if self.error:
                raise TestProcessError(self, "check_kill found error output")
        else:
            raise TestProcessError(self, "check_kill process not running")

    def expect(self, pattern):
        line = self.stdout.readline()
        match = re.search(pattern, line)
        if not match:
            raise TestProcessError(self, "can't find '%s' in '%s'" % (pattern, line))
        return match

    @property
    def error(self):
        """Return stderr as string, may only be used after process has terminated."""
        assert(self.poll is not None)
        if not hasattr(self, "_error"):
            self.errfile.close()  # Not auto-deleted
            with open(self.errfile.name) as f:  # Re-open to read
                self._error = f.read().strip()
            os.unlink(self.errfile.name)
        return self._error

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.on_exit()


def check_output(*args, **kwargs):
    return Popen(*args, **kwargs).communicate()[0]


class Server(Popen):
    """A process that prints 'listening on <port>' to stdout"""

    def __init__(self, *args, **kwargs):
        super(Server, self).__init__(*args, **kwargs)
        self.port = self.expect("listening on ([0-9]+)$").group(1)
